@@ -25,6 +25,7 @@ import subprocess
 import sys
 import typing
 import abc
+import warnings
 from datetime import datetime
 import enum
 from typing import Optional, Tuple, Union, List
@@ -35,7 +36,7 @@ import tifffile as tf
 from scipy import io as spio
 from pwspy.analysis import AbstractHDFAnalysisResults
 from pwspy.dataTypes import _jsonSchemasPath
-from pwspy.dataTypes._other import CameraCorrection, Roi
+from pwspy.dataTypes._other import CameraCorrection, Roi, RoiFile
 import pwspy.dataTypes._data as pwsdtd
 from pwspy import dateTimeFormat
 from pwspy.utility.misc import cached_property
@@ -176,7 +177,7 @@ class AnalysisManager(abc.ABC):
         """
 
         Returns:
-            AbstractHDFAnalysisResults: The class that contains analysis results for this acquisition type
+            The class that is used to contain analysis results for this acquisition type.
         """
         pass
 
@@ -206,17 +207,18 @@ class AnalysisManager(abc.ABC):
         else:
             return []
 
-    def saveAnalysis(self, analysis: AbstractHDFAnalysisResults, name: str):
+    def saveAnalysis(self, analysis: AbstractHDFAnalysisResults, name: str, overwrite: bool = False):
         """
 
         Args:
             analysis: An AnalysisResults object to be saved.
             name: The name to save the analysis as
+            overwrite: If `True` then any existing file of the same name will be replaced. If `False` an exception will be raised.
         """
         path = os.path.join(self.__filePath, 'analyses')
         if not os.path.exists(path):
             os.mkdir(path)
-        analysis.toHDF(path, name)
+        analysis.toHDF(path, name, overwrite=overwrite)
 
     def loadAnalysis(self, name: str) -> AbstractHDFAnalysisResults:
         """
@@ -770,12 +772,17 @@ class AcqDir:
         directory: the file path the root directory of the acquisition
     """
     def __init__(self, directory: str):
-        self.filePath = directory #TODO should this be forced to an absolute path? Doesn't appear to be causing any problems like it is now.
+        self.filePath = os.path.abspath(directory)  # Forcing an absolute path helps by: A: normalizing the path so string comparisons work. B: making sure that if the object is pickled and then unpickled from a different working direcotory the path will still be valid if the data is unmoved.
         if (self.pws is None) and (self.dynamics is None) and (len(self.fluorescence) == 0):
             raise OSError(f"Could not find a valid PWS or Dynamics Acquisition at {directory}.")
 
     def __repr__(self):
-        return f"AcqDir({self.filePath})"
+        if len(self.filePath) > 20:
+            path = ("%.15s" % self.filePath[::-1])[::-1]  # This is confusing. We cut everything but the last characters to make long paths readable.
+            path = "..." + path  # Add ellipsis
+        else:
+            path = self.filePath
+        return f"{self.__class__.__name__}({path})"
 
     @cached_property
     def pws(self) -> Optional[ICMetaData]:
@@ -830,31 +837,34 @@ class AcqDir:
         else: #We must have one of these two items.
             return self.dynamics.idTag
 
-    def getRois(self) -> List[Tuple[str, int, Roi.FileFormats]]:
+    def getRois(self) -> List[Tuple[str, int, RoiFile.FileFormats]]:
         """Return information about the Rois found in the acquisition's file path.
         See documentation for Roi.getValidRoisInPath()"""
         assert self.filePath is not None
-        return Roi.getValidRoisInPath(self.filePath)
+        return RoiFile.getValidRoisInPath(self.filePath)
 
-    def loadRoi(self, name: str, num: int, fformat: Roi.FileFormats = None) -> Roi:
+    def loadRoi(self, name: str, num: int, fformat: RoiFile.FileFormats = None) -> RoiFile:
         """Load a Roi that has been saved to file in the acquisition's file path."""
-        assert isinstance(name, str)
-        assert isinstance(num, int)
-        if fformat == Roi.FileFormats.MAT:
-            return Roi.fromMat(self.filePath, name, num)
-        elif fformat == Roi.FileFormats.HDF2:
-            return Roi.fromHDF(self.filePath, name, num)
-        elif fformat == Roi.FileFormats.HDF:
-            return Roi.fromHDF_legacy(self.filePath, name, num)
-        else:
-            return Roi.loadAny(self.filePath, name, num)
 
-    def saveRoi(self, roi: Roi, overwrite: bool = False) -> None:
+        assert isinstance(name, str), f"The ROI name must be a string. Got a {type(name)}"
+        assert isinstance(num, int), f"The ROI number must be an integer. Got a {type(num)}"
+        if fformat == RoiFile.FileFormats.MAT:
+            return RoiFile.fromMat(self.filePath, name, num, acquisition=self)
+        elif fformat == RoiFile.FileFormats.HDF3:
+            return RoiFile.fromHDF(self.filePath, name, num, acquisition=self)
+        elif fformat == RoiFile.FileFormats.HDF2:
+            return RoiFile.fromHDF_legacy(self.filePath, name, num, acquisition=self)
+        elif fformat == RoiFile.FileFormats.HDF:
+            return RoiFile.fromHDF_legacy_legacy(self.filePath, name, num, acquisition=self)
+        else:
+            return RoiFile.loadAny(self.filePath, name, num, acquisition=self)
+
+    def saveRoi(self, roiName: str, roiNumber: int, roi: Roi, overwrite: bool = False) -> RoiFile:
         """Save a Roi to file in the acquisition's file path."""
-        roi.toHDF(self.filePath, overwrite=overwrite)
+        return RoiFile.toHDF(roi, roiName, roiNumber, self.filePath, overwrite=overwrite, acquisition=self)
 
     def deleteRoi(self, name: str, num: int):
-        Roi.deleteRoi(self.filePath, name, num)
+        RoiFile.deleteRoi(self.filePath, name, num)
 
     def editNotes(self):
         """Create a `notes.txt` file if it doesn't already exists and open it in a text editor."""
@@ -894,7 +904,19 @@ class AcqDir:
     def getNumber(self) -> int:
         return int(self.filePath.split("Cell")[-1])
 
-    
+    def __setstate__(self, state):
+        """When the object is unpickled this will check if the file path is still valid."""
+        if not os.path.exists(state['filePath']):
+            warnings.warn(f"{self.__class__.__name__} file path cannot be found. {state['filePath']}")
+        self.__dict__ = state  # This is the default thing to do to unpicle the object.
+
+    def __hash__(self) -> int:
+        return hash(self.filePath)  # Since the class is intimiately tied to the filesystem this is all that makes it unique
+
+    def __eq__(self, other: AcqDir) -> bool:
+        if not isinstance(other, AcqDir):
+            return False
+        return self.filePath == other.filePath
 
 if __name__ == '__main__':
     md = ICMetaData.fromNano(r'C:\Users\nicke\Desktop\LTL20b_Tracking cells in 50%EtOH,95%EtOH,Water\95% ethanol\Cell1')
\ No newline at end of file
