@@ -23,23 +23,22 @@ Modules
 .. autosummary::
    :toctree: generated/
 
-   GoogleDriveDownloader
+   acquisition
+   DConversion
    fileIO
-   misc
-   machineVision
    fluorescence
+   machineVision
+   micromanager
+   misc
    plotting
    reflection
-   micromanager
-   matplotlibWidgets
-   DConversion
+
 
 """
 
-from ._GoogleDriveDownloader import GoogleDriveDownloader
 import os
 
 thinFilmPath = os.path.join(os.path.split(__file__)[0], 'thinFilmInterferenceFiles')
 
-__all__ = ['GoogleDriveDownloader', 'fileIO', 'misc', 'machineVision', 'fluorescence', 'plotting', 'reflection',
+__all__ = ['fileIO', 'misc', 'machineVision', 'fluorescence', 'plotting', 'reflection',
            'micromanager', 'DConversion']
